<?php

namespace Urbanoppc\Shipping\Helper;

class Urbano extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * Numero de Shipper
     *
     * @var int|string
     */
    protected $shipper;

    protected $_cart;

    /**
     * Clave de acceso del shipper
     *
     * @var string
     */
    protected $password;
    
    /**
     * Ruta base de API Urbanoppc
     *
     * @var string
     */
    protected $base_url;

    /**
     * EndPoints disponibles para consumo
     *
     * @var array
     */
    protected $endpoints;

    /** @var \Urbanoppc\Shipping\Helper\Config */
    protected $_config;

    /** @var \Urbano\Shipping\Helper\Config */
    protected $_config2;

    /** @var \Magento\Framework\HTTP\ZendClientFactory */
    protected $_httpClient;

    /** @var \Psr\Log\LoggerInterface */
    protected $_logger;

    /**
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\HTTP\ZendClientFactory $httpClient
     * @param \Urbanoppc\Moduleshipping\Helper\Config $config
     * @param \Urbano\Moduleshipping\Helper\Config $config2
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\HTTP\ZendClientFactory $httpClient,
        \Magento\Checkout\Model\Cart $cartModel,
        \Urbanoppc\Shipping\Helper\Config $config,
        \Urbano\Shipping\Helper\Config $config2


    ) {
        $this->_config = $config->getConfig();
        $this->_config2 = $config2->getConfig();

        $this->shipper = $this->_config['shipper'];
        $this->password = $this->_config['key'];
        $this->base_url = $this->_config2['gateway'];
        $this->_cart = $cartModel;
        $this->endpoints = [
            "tarifa" => "/consulta_tarifa_rest/",
            "carga_clientes" => "/cargaCliente/",
            "tracking" => "/cespecifica/",
            "tracking_masivo" => "/tracking/consultaTrackingMasivo/",
            "impresion_etiqueta" => "/impresionEtiqueta/"
        ];

        $this->_httpClient = $httpClient;
        $this->_logger = $logger;
    }


    /**
     * Devuelve los EndPoints disponibles.
     *
     * @return array
     */
    public function getEndPoints()
    {
        return $this->endpoints;
    }

    /**
     * Tarifa de Urbanoppc
     *
     * Obtiene mediante API la tarifa de Urbanoppc de acuerdo al
     * codigo postal enviado.
     *
     * @param int|string $zipcode
     * @return array
     */
    public function postTarifa($zipcode)
    {
        try {
            $headers = [
                "content-type" => "application/json",
            ];
            
            // Traigo el carrito
            $items = $this->_cart->getQuote()->getAllItems();
            $weight = 0;
            //itero para obtener el peso
            foreach($items as $item) {
                $weight += ($item->getWeight() * $item->getQty()) ;
                $height =  0;
                $width =  0;
                $length =  0;

            }

            $params = [
                "codigoPostal" => $zipcode,
                "pesoEspecifico" => $weight,
                "pesoVolumetrico" => 0,
                "alto" => $height,
                "largo" => $height,
                "ancho" => $width,
                "autentificacion" => [
                    "shipper" => $this->shipper,
                    "password" => $this->password,
                ]
            ];

            $uri = "{$this->base_url}{$this->endpoints['tarifa']}";
            $request = $this->caller($uri, \Zend_Http_Client::POST, json_encode($params), $headers);
            $requestBody = json_decode($request->getBody(), true);

            $this->_logger->log(100, print_r($request, true));
            $this->_logger->log(100, print_r($requestBody, true));
            
            if ($request->getStatus() != 200 || $requestBody['codError'] != 0) {
                $response = array_merge(["success" => "false", "code" => $request->getStatus()], $requestBody);
                
                $this->_logger->log(100, print_r($response, true));
                return $response;
            }

            $response = array_merge(["success" => "true", "code" => $request->getStatus()], $requestBody);
            
            $this->_logger->log(100, print_r($params, true));
            return $response;
        } catch (\Exception $e) {
            return [
                "success" => false,
                "code" => $e->getCode(),
                "message" => $e->getMessage(),
            ];
        }
    }

    /**
     * Genera una nueva guia.
     *
     * Envia el/los productos a Urbanoppc para generar una nueva guia.
     *
     * @param array $body
     * @return object
     */
    public function postCargaClientes($body)
    {
        try {
            $headers = [
                "content-type" => "application/json",
            ];

            $uri = "{$this->base_url}{$this->endpoints['carga_clientes']}";
            $request = $this->caller($uri, \Zend_Http_Client::POST, json_encode($body), $headers);
            $requestBody = json_decode($request->getBody(), true);
            
            $this->_logger->log(100, print_r($request, true));
            $this->_logger->log(100, print_r($requestBody, true));

            if ($request->getStatus() != 200 || $requestBody['resultado'] == "error") {
                $response = array_merge(["success" => "false", "code" => $request->getStatus()], $requestBody);

                $this->_logger->log(100, print_r($response, true));
                return $response;
            }

            $response = array_merge(["success" => "true", "code" => $request->getStatus()], $requestBody);

            $this->_logger->log(100, print_r($response, true));
            return $response;
        } catch (\Exception $e) {
            return [
                "success" => (bool) false,
                "code" => $e->getCode(),
                "message" => $e->getMessage(),
            ];
        }
    }

    public function postTracking($guias)
    {
        try {
            $headers = [
                "content-type" => "application/json",
            ];

            $body = [
                "autentificacion"=> [
                    "shipper" => $this->shipper,
                    "password" => $this->password
                ],
            ];
            array_map(function ($guia) {
                $body['datos'][] = [
                    "codigoSeguimiento"=> $guia['codigoSeguimiento'],
                    "codigoAlternativo"=> $guia['codigoAlternativo']
                ];
            }, $guias);

            $uri = "{$this->base_url}{$this->endpoints['tracking_masivo']}";
            $request = $this->caller($uri, \Zend_Http_Client::POST, json_encode($body), $headers);
            $requestBody = json_decode($request->getBody(), true);
            $this->_logger->log(100, print_r($request, true));

            if ($request->getStatus() != 200 || $requestBody['resultado'] == "error") {
                $response = array_merge(["success" => "false", "code" => $request->getStatus()], $requestBody);
                
                $this->_logger->log(100, print_r($response, true));
                return $response;
            }

            $response = array_merge(["success" => "true", "code" => $request->getStatus()], $requestBody);
            
            $this->_logger->log(100, print_r($response, true));
            return $response;
        } catch (\Exception $e) {
            return [
                "success" => false,
                "code" => $e->getCode(),
                "message" => $e->getMessage(),
            ];
        }
    }

    /**
     * Hace la peticion a la API Rest de Urbanoppc
     *
     * @param string $url
     * @param string $method
     * @param object $jsonBody
     * @param array $headers
     * @return object
     */
    private function caller($url, $method, $jsonBody, $headers = [])
    {
        $api = $this->_httpClient->create();
        $api->setUri($url);
        $api->setMethod($method);
        $api->setHeaders($headers);

        if ($method == \Zend_Http_Client::POST) {
            $api->setRawData($jsonBody);
        }

        return $api->request();
    }

    /**
     * Prepara el cuerpo para la solicitud a CargaClientes de Urbanoppc
     *
     * @param \Magento\Sales\Model\Order $order
     *
     * @return array
     */
    public function prepareBodyPostCargaClientes($order)
    {
        $address = $order->getShippingAddress();
        $uea_sd_cod_poe = null;
        $observaciones = "Codigo POE";
        $servicio = "T";
        $body = [
            'autentificacion' => [
                'shipper' => $this->_config['shipper'],
                'password' => $this->_config['key'],
                'uea_sd_shipper_id' => $this->_config['shipper'],
                'uea_sd_shipper_pass' => $this->_config['key'],
                'uea_sd_cod_poe' => $this->_config['poe'],
            ],
            'destinatario' => [
                'tipoDocumento' => '',
                'numeroDocumento' => '',
                'nombre' => $address->getName(),
                'email' => [
                    $address->getEmail(),
                ],
                'telefono' => $address->getTelephone(),
                'celular' => '',
            ],
            'autorizado' => [
                [
                    'tipoDocumento' => '',
                    'numeroDocumento' => '',
                    'nombre' => '',
                    'email' => [],
                    'telefono' => '',
                    'celular' => '',
                ],
            ],
            'domicilio' => [
                'direccion' => $address->getData('street'),
                'altura' => '',
                'piso' => '',
                'departamento' => '',
                'codigoPostal' => $address->getPostcode(),
                'localidad' => $address->getCity(),
                'provincia' => $address->getRegion(),
                'latitud' => '',
                'longitud' => '',
                'telefono' => [],
            ],
            'domicilioAlternativo' => [
                'direccion' => '',
                'altura' => '',
                'piso' => '',
                'departamento' => '',
                'codigoPostal' => '',
                'localidad' => '',
                'provincia' => '',
                'latitud' => '',
                'longitud' => '',
                'telefono' => [],
            ],
            'datoNumerico' => '',
            'codigoSeguimiento' => $order->getIncrementId(),
            'codigoAlternativo' => '',
            'modeloSms' => $this->_config['modeloSms'],
            'modeloEmail' => $this->_config['modeloEmail'],
            'servicio' => 'E', // Entrega
            'marcaDeAgua' => '',
            'remito' => '',
            'observaciones' => $observaciones,
        ];

        if (
            !empty($this->_config['activeContraReembolso']) &&
            $order->getPayment()->getMethod() == $this->_config['activeContraReembolso']
        ) {
            $body['servicio'] = 'B';
        }

        $body['productos'] = $this->addProductsToPostBody($order, $body['servicio']);

        return $body;
    }


    /**
     * Cree en contenido de Productos para la solicitud a Urbanoppc.
     *
     * @param \Magento\Sales\Model\Order $order
     * @param string $servicio
     *
     * @return array
     */
    protected function addProductsToPostBody($order, $servicio)
    {
        $productos = [];

        switch ($servicio) {
            case 'E':
                /** @var \Magento\Sales\Model\Order\Item $orderItem */
                foreach ($order->getAllItems() as $orderItem) {
                    if (!$orderItem->getQtyToShip() || $orderItem->getIsVirtual()) {
                        continue;
                    }

                    $productos[] = [
                        'largo' => 0,
                        'alto' => 0,
                        'ancho' => 0,
                        'peso' => $orderItem->getWeight(),
                        'valor' => 0,
                        'valorContrareembolso' => 0,
                        'cantidad' => 1,  // Pensado siempre como cross
                        'sku' => '', // Si es warehousing, en caso de crossdocking va vacio
                        'descripcionProducto' => $order->getIncrementId(),
                    ];
                }
            break;

            case 'B':
                /** @var \Magento\Sales\Model\Order\Item $orderItem */
                foreach ($order->getAllItems() as $orderItem) {
                    if (!$orderItem->getQtyToShip() || $orderItem->getIsVirtual()) {
                        continue;
                    }

                    $productos[] = [
                        'largo' => 0,
                        'alto' => 0,
                        'ancho' => 0,
                        'peso' => $orderItem->getWeight(),
                        'valor' => 0,
                        'valorContrareembolso' => $orderItem->getPrice(),
                        'cantidad' => 1,  // Pensado siempre como cross
                        'sku' => '', // Si es warehousing, en caso de crossdocking va vacio
                        'descripcionProducto' => $order->getIncrementId(),
                    ];
                }
            break;
        }

        return $productos;
    }
}
